package ru.ifsoft.network;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.IdRes;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.android.volley.toolbox.ImageLoader;
import com.google.android.gms.ads.MobileAds;
import com.mikhaellopez.circularimageview.CircularImageView;

import ru.ifsoft.network.app.App;
import ru.ifsoft.network.common.ActivityBase;
import ru.ifsoft.network.dialogs.CoverChooseDialog;
import ru.ifsoft.network.dialogs.FriendRequestActionDialog;
import ru.ifsoft.network.dialogs.MyPhotoActionDialog;
import ru.ifsoft.network.dialogs.MyPostActionDialog;
import ru.ifsoft.network.dialogs.PeopleNearbySettingsDialog;
import ru.ifsoft.network.dialogs.PeopleNearbySexDialog;
import ru.ifsoft.network.dialogs.PhotoChooseDialog;
import ru.ifsoft.network.dialogs.PhotoDeleteDialog;
import ru.ifsoft.network.dialogs.PostActionDialog;
import ru.ifsoft.network.dialogs.PostDeleteDialog;
import ru.ifsoft.network.dialogs.PostReportDialog;
import ru.ifsoft.network.dialogs.PostShareDialog;
import ru.ifsoft.network.dialogs.ProfileBlockDialog;
import ru.ifsoft.network.dialogs.ProfileReportDialog;
import ru.ifsoft.network.dialogs.SearchSettingsDialog;

public class MainActivity extends ActivityBase implements PhotoChooseDialog.AlertPositiveListener, CoverChooseDialog.AlertPositiveListener, ProfileReportDialog.AlertPositiveListener, ProfileBlockDialog.AlertPositiveListener, PostDeleteDialog.AlertPositiveListener, PostReportDialog.AlertPositiveListener, MyPostActionDialog.AlertPositiveListener, PostActionDialog.AlertPositiveListener, PostShareDialog.AlertPositiveListener, PhotoDeleteDialog.AlertPositiveListener, MyPhotoActionDialog.AlertPositiveListener, PeopleNearbySettingsDialog.AlertPositiveListener, SearchSettingsDialog.AlertPositiveListener, FriendRequestActionDialog.AlertPositiveListener, PeopleNearbySexDialog.AlertPositiveListener {

    Toolbar mToolbar;

    private TextView mNavHeaderFullname, mNavHeaderUsername;
    private CircularImageView mNavHeaderPhoto, mNavHeaderIcon;
    private ImageView mNavHeaderCover;
    private DrawerLayout mDrawerLayout;
    private NavigationView mNavView;
    public Menu mNavMenu;
    private View mNavHeaderLayout;

    // used to store app title
    private CharSequence mTitle;

    LinearLayout mContainerAdmob;

    Fragment fragment;
    Boolean action = false;
    int page = 0;

    private Boolean signup = false;

    private Boolean restore = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        // Initialize Google Admob

        MobileAds.initialize(this, getString(R.string.admob_app_id));

        if (savedInstanceState != null) {

            //Restore the fragment's instance
            fragment = getSupportFragmentManager().getFragment(savedInstanceState, "currentFragment");

            restore = savedInstanceState.getBoolean("restore");
            mTitle = savedInstanceState.getString("mTitle");
            signup = savedInstanceState.getBoolean("signup");

        } else {

            fragment = new FeedFragment();

            restore = false;
            mTitle = getString(R.string.app_name);

            signup = false;
        }

        if (fragment != null) {

            FragmentManager fragmentManager = getSupportFragmentManager();
            fragmentManager.beginTransaction().replace(R.id.container_body, fragment).commit();
        }

        Intent i = getIntent();

        signup = i.getBooleanExtra("signup", false);

        mToolbar = (Toolbar) findViewById(R.id.toolbar);

        setSupportActionBar(mToolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
        getSupportActionBar().setTitle(mTitle);

        mDrawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);

        initDrawerMenu();

        mContainerAdmob = (LinearLayout) findViewById(R.id.container_admob);

        mContainerAdmob.setVisibility(View.GONE);

        if (!restore) {

            // nav_feed - FEED SECTION
            // nav_stream - STREAM SECTION
            // nav_popular - POPULAR SECTION
            // nav_notifications - NOTIFICATIONS SECTION
            // nav_messages - MESSAGES SECTION

            // FOR OTHERS INDEXES: SEE displayFragment FUNCTION and "res/menu/menu_drawer.xml" file

            displayFragment(mNavMenu.findItem(R.id.nav_feed).getItemId(), mNavMenu.findItem(R.id.nav_feed).getTitle().toString());
        }
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {

        super.onSaveInstanceState(outState);

        outState.putBoolean("restore", true);
        outState.putBoolean("signup", signup);
        outState.putString("mTitle", getSupportActionBar().getTitle().toString());
        getSupportFragmentManager().putFragment(outState, "currentFragment", fragment);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {

        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        fragment.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    public void onChangeDistance(int position) {

        PeopleNearbyFragment p = (PeopleNearbyFragment) fragment;
        p.onChangeDistance(position);
    }

    @Override
    public void onChangeSex(int position) {

        PeopleNearbyFragment p = (PeopleNearbyFragment) fragment;
        p.onChangeSex(position);
    }

    @Override
    public void onCloseSettingsDialog(int searchGender, int searchOnline, int searchPhoto) {

        SearchFragment p = (SearchFragment) fragment;
        p.onCloseSettingsDialog(searchGender, searchOnline, searchPhoto);
    }

    @Override
    public void onAcceptRequest(int position) {

        NotificationsFragment p = (NotificationsFragment) fragment;
        p.onAcceptRequest(position);
    }

    @Override
    public void onRejectRequest(int position) {

        NotificationsFragment p = (NotificationsFragment) fragment;
        p.onRejectRequest(position);
    }

    @Override
    public void onPhotoFromGallery() {

        ProfileFragment p = (ProfileFragment) fragment;
        p.photoFromGallery();
    }

    @Override
    public void onPhotoFromCamera() {

        ProfileFragment p = (ProfileFragment) fragment;
        p.photoFromCamera();
    }

    @Override
    public void onCoverFromGallery() {

        ProfileFragment p = (ProfileFragment) fragment;
        p.coverFromGallery();
    }

    @Override
    public void onCoverFromCamera() {

        ProfileFragment p = (ProfileFragment) fragment;
        p.coverFromCamera();
    }

    @Override
    public void onProfileReport(int position) {

        ProfileFragment p = (ProfileFragment) fragment;
        p.onProfileReport(position);
    }

    @Override
    public void onProfileBlock() {

        ProfileFragment p = (ProfileFragment) fragment;
        p.onProfileBlock();
    }

    @Override
    public void onPostRePost(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostRePost(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostRePost(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostRePost(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostRePost(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostRePost(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostShare(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostShare(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostShare(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostShare(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostShare(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostShare(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostDelete(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostDelete(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostDelete(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostDelete(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostDelete(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostDelete(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostRemove(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostRemove(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostRemove(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostRemove(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostRemove(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostRemove(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostEdit(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostEdit(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostEdit(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostEdit(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostEdit(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostEdit(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostCopyLink(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostCopyLink(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostCopyLink(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostCopyLink(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostCopyLink(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostCopyLink(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostReportDialog(int position) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.report(position);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.report(position);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.report(position);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.report(position);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.report(position);

                break;
            }

            default: {

                break;
            }
        }
    }

    @Override
    public void onPostReport(int position, int reasonId) {

        switch (page) {

            case 1: {

                ProfileFragment p = (ProfileFragment) fragment;
                p.onPostReport(position, reasonId);

                break;
            }

            case 3: {

                FeedFragment p = (FeedFragment) fragment;
                p.onPostReport(position, reasonId);

                break;
            }

            case 4: {

                StreamFragment p = (StreamFragment) fragment;
                p.onPostReport(position, reasonId);

                break;
            }

            case 5: {

                PopularFragment p = (PopularFragment) fragment;
                p.onPostReport(position, reasonId);

                break;
            }

            case 9: {

                FavoritesFragment p = (FavoritesFragment) fragment;
                p.onPostReport(position, reasonId);

                break;
            }

            default: {

                break;
            }
        }
    }

    private void initDrawerMenu() {

        mNavView = (NavigationView) findViewById(R.id.nav_view);

        final DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);

        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(this, drawer, mToolbar, R.string.nav_view_open, R.string.nav_view_close) {

            public void onDrawerOpened(View drawerView) {

                refreshMenu();

                super.onDrawerOpened(drawerView);
            }
        };

        mDrawerLayout.addDrawerListener(toggle);
        toggle.syncState();

        mNavView.setNavigationItemSelectedListener(new NavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(final MenuItem menuItem) {

                displayFragment(menuItem.getItemId(), menuItem.getTitle().toString());

                mDrawerLayout.closeDrawers();

                return true;
            }
        });

        mNavMenu = mNavView.getMenu();

        mNavView.setItemIconTintList(getResources().getColorStateList(R.color.nav_state_list));

        mNavHeaderLayout = mNavView.getHeaderView(0);
        mNavHeaderFullname = (TextView) mNavHeaderLayout.findViewById(R.id.userFullname);
        mNavHeaderUsername = (TextView) mNavHeaderLayout.findViewById(R.id.userUsername);

        mNavHeaderPhoto = (CircularImageView) mNavHeaderLayout.findViewById(R.id.userPhoto);
        mNavHeaderIcon = (CircularImageView) mNavHeaderLayout.findViewById(R.id.verified);
        mNavHeaderCover = (ImageView) mNavHeaderLayout.findViewById(R.id.userCover);
    }

    private void refreshMenu() {

        if (App.getInstance().getVerify() == 1) {

            mNavHeaderIcon.setVisibility(View.VISIBLE);

        } else {

            mNavHeaderIcon.setVisibility(View.GONE);
        }

        if (App.getInstance().getId() > 0) {

            updateNavItemCounter(mNavView, R.id.nav_friends, App.getInstance().getNewFriendsCount());
            updateNavItemCounter(mNavView, R.id.nav_notifications, App.getInstance().getNotificationsCount());
            updateNavItemCounter(mNavView, R.id.nav_messages, App.getInstance().getMessagesCount());
            updateNavItemCounter(mNavView, R.id.nav_guests, App.getInstance().getGuestsCount());

            mNavHeaderCover.setScaleType(ImageView.ScaleType.CENTER_CROP);

            mNavHeaderFullname.setVisibility(View.VISIBLE);
            mNavHeaderUsername.setVisibility(View.VISIBLE);
            mNavHeaderPhoto.setVisibility(View.VISIBLE);

            mNavHeaderFullname.setText(App.getInstance().getFullname());
            mNavHeaderUsername.setText("@" + App.getInstance().getUsername());

            if (App.getInstance().getPhotoUrl() != null && App.getInstance().getPhotoUrl().length() > 0) {

                ImageLoader imageLoader = App.getInstance().getImageLoader();

                imageLoader.get(App.getInstance().getPhotoUrl(), ImageLoader.getImageListener(mNavHeaderPhoto, R.drawable.profile_default_photo, R.drawable.profile_default_photo));

            } else {

                mNavHeaderPhoto.setImageResource(R.drawable.profile_default_photo);
            }

            if (App.getInstance().getCoverUrl() != null && App.getInstance().getCoverUrl().length() > 0) {

                ImageLoader imageLoader = App.getInstance().getImageLoader();

                imageLoader.get(App.getInstance().getCoverUrl(), ImageLoader.getImageListener(mNavHeaderCover, R.drawable.profile_default_cover, R.drawable.profile_default_cover));

            } else {

                mNavHeaderCover.setImageResource(R.drawable.profile_default_cover);
            }

        } else {

            mNavHeaderFullname.setVisibility(View.GONE);
            mNavHeaderUsername.setVisibility(View.GONE);
            mNavHeaderPhoto.setVisibility(View.GONE);

            mNavHeaderCover.setScaleType(ImageView.ScaleType.CENTER_CROP);
            mNavHeaderCover.setImageResource(R.drawable.profile_default_cover);
        }
    }

    private void updateNavItemCounter(NavigationView nav, @IdRes int itemId, int count) {

        TextView view = (TextView) nav.getMenu().findItem(itemId).getActionView().findViewById(R.id.counter);
        view.setText(String.valueOf(count));

        if (count <= 0) {

            view.setVisibility(View.GONE);

        } else {

            view.setVisibility(View.VISIBLE);
        }
    }

    public void displayFragment(int id, String title) {

        action = false;

        switch (id) {

            case R.id.nav_profile: {

                page = 1;

                fragment = new ProfileFragment();
                getSupportActionBar().setTitle(R.string.page_1);

                action = true;

                break;
            }

            case R.id.nav_gallery: {

                page = 2;

                fragment = new GalleryFragment();
                getSupportActionBar().setTitle(R.string.page_11);

                action = true;

                break;
            }

            case R.id.nav_feed: {

                if (signup) {

                    page = 4;

                    fragment = new StreamFragment();
                    getSupportActionBar().setTitle(R.string.page_3);

                    signup = false;

                } else {

                    page = 3;

                    fragment = new FeedFragment();
                    getSupportActionBar().setTitle(R.string.page_2);
                }

                action = true;

                break;
            }

            case R.id.nav_stream: {

                page = 4;

                fragment = new StreamFragment();
                getSupportActionBar().setTitle(R.string.page_3);

                action = true;

                break;
            }

            case R.id.nav_popular: {

                page = 5;

                fragment = new PopularFragment();
                getSupportActionBar().setTitle(R.string.page_4);

                action = true;

                break;
            }

            case R.id.nav_friends: {

                page = 6;

                fragment = new FriendsFragment();
                getSupportActionBar().setTitle(R.string.page_5);

                action = true;

                break;
            }

            case R.id.nav_notifications: {

                page = 7;

                fragment = new NotificationsFragment();
                getSupportActionBar().setTitle(R.string.page_6);

                action = true;

                break;
            }

            case R.id.nav_messages: {

                page = 8;

                fragment = new DialogsFragment();
                getSupportActionBar().setTitle(R.string.page_7);

                action = true;

                break;
            }

            case R.id.nav_favorites: {

                page = 9;

                fragment = new FavoritesFragment();
                getSupportActionBar().setTitle(R.string.page_8);

                action = true;

                break;
            }

            case R.id.nav_guests: {

                page = 10;

                fragment = new GuestsFragment();
                getSupportActionBar().setTitle(R.string.page_12);

                action = true;

                break;
            }

            case R.id.nav_groups: {

                page = 11;

                fragment = new GroupsFragment();
                getSupportActionBar().setTitle(R.string.page_9);

                action = true;

                break;
            }

            case R.id.nav_upgrades: {

                page = 12;

                fragment = new UpgradesFragment();
                getSupportActionBar().setTitle(R.string.page_14);

                action = true;

                break;
            }

            case R.id.nav_nearby: {

                page = 13;

                fragment = new PeopleNearbyFragment();
                getSupportActionBar().setTitle(R.string.page_15);

                action = true;

                break;
            }

            case R.id.nav_market: {

                page = 14;

                fragment = new MarketFragment();
                getSupportActionBar().setTitle(R.string.page_16);

                action = true;

                break;
            }

            case R.id.nav_search: {

                page = 15;

                fragment = new SearchFragment();
                getSupportActionBar().setTitle("");

                action = true;

                break;
            }

            default: {

                Intent i = new Intent(MainActivity.this, SettingsActivity.class);
                startActivity(i);
            }
        }

        if (action && fragment != null) {

            getSupportActionBar().setDisplayShowCustomEnabled(false);
            getSupportActionBar().setDisplayShowTitleEnabled(true);
            getSupportActionBar().setTitle(title);

            FragmentManager fragmentManager = getSupportFragmentManager();
            fragmentManager.beginTransaction().replace(R.id.container_body, fragment).commit();
        }
    }

    @Override
    public void onPhotoDelete(int position) {

        GalleryFragment p = (GalleryFragment) fragment;
        p.onPhotoDelete(position);
    }

    @Override
    public void onPhotoRemoveDialog(int position) {

        GalleryFragment p = (GalleryFragment) fragment;
        p.onPhotoRemove(position);
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        // If the nav drawer is open, hide action items related to the content view
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {

        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        switch (item.getItemId()) {

            case android.R.id.home: {

                return true;
            }

            default: {

                return super.onOptionsItemSelected(item);
            }
        }
    }

    @Override
    public void onBackPressed() {

        super.onBackPressed();
    }

    @Override
    protected void onResume() {

        super.onResume();

        refreshMenu();
    }

    @Override
    public void setTitle(CharSequence title) {

        mTitle = title;
        getSupportActionBar().setTitle(mTitle);
    }

    public void hideAds() {

        if (App.getInstance().getAdmob() == ADMOB_DISABLED) {

            mContainerAdmob.setVisibility(View.GONE);
        }
    }
}
