<?php

/*!
 * https://raccoonsquare.com
 * raccoonsquare@gmail.com
 *
 * Copyright 2012-2021 Demyanchuk Dmitry (raccoonsquare@gmail.com)
 */;

if (!defined("APP_SIGNATURE")) {

    header("Location: /");
    exit;
}

require_once '../sys/addons/firebase/autoload.php';

use Kreait\Firebase\Factory;

use Firebase\Auth\Token\Exception\InvalidToken;
use Kreait\Firebase\ServiceAccount;

if (!empty($_POST)) {

    $accountId = isset($_POST['accountId']) ? $_POST['accountId'] : 0;
    $accessToken = isset($_POST['accessToken']) ? $_POST['accessToken'] : '';

    $phoneNumber = isset($_POST['phoneNumber']) ? $_POST['phoneNumber'] : '';

    $idToken = isset($_POST['token']) ? $_POST['token'] : '';

    $phoneNumber = helper::clearText($phoneNumber);
    $phoneNumber = helper::escapeText($phoneNumber);

    $result = array(
        "error" => true,
        "error_code" => ERROR_UNKNOWN,
        "token" => "",
        "verified" => false
    );

    $auth = new auth($dbo);

    if (!$auth->authorize($accountId, $accessToken)) {

        api::printError(ERROR_ACCESS_TOKEN, "Error authorization.");
    }

    $jsonFileName = "";

    if ($files = glob("js/firebase/*.json")) {

        $jsonFileName = $files[0];
    }

    $serviceAccount = ServiceAccount::fromValue($jsonFileName);

    $firebase = (new Factory)->withServiceAccount($serviceAccount);

    $firebaseAuth = $firebase->createAuth();

    try {

        $token = $firebaseAuth->verifyIdToken($idToken, true);

        $user = $firebaseAuth->getUser($token->getClaim("user_id"));

        if ($user->phoneNumber != null) {

            if (!$helper->isPhoneNumberExists($user->phoneNumber)) {

                $account = new account($dbo, $accountId);
                $account->updateOtpVerification($user->phoneNumber, 1);

                if (BONUS_OTP_VERIFICATION != 0) {

                    $account->setBalance($account->getBalance() + BONUS_OTP_VERIFICATION);

                    $payments = new payments($dbo);
                    $payments->setRequestFrom($accountId);
                    $payments->create(PA_BUY_OTP_VERIFICATION, PT_BONUS, BONUS_OTP_VERIFICATION);
                    unset($payments);
                }

                if (auth::getCurrentUserId() != 0) {

                    auth::setCurrentUserOtpVerified(1);
                }

                $result = array(
                    "error" => false,
                    "error_code" => ERROR_SUCCESS,
                    "verified" => true
                );

            }  else {

                $result = array(
                    "error" => true,
                    "error_code" => ERROR_OTP_PHONE_NUMBER_TAKEN,
                    "verified" => false
                );
            }
        }

        $firebaseAuth->revokeRefreshTokens($token->getClaim("user_id"));

    } catch (InvalidToken $e) {

        $result['token'] = $e->getMessage();

    } catch (\Kreait\Firebase\Exception\AuthException $e) {

        $result['token'] = $e->getMessage();

    } catch (\Kreait\Firebase\Exception\FirebaseException $e) {

        $result['token'] = $e->getMessage();
    }

    echo json_encode($result);
    exit;
}
